
# q-expansion of newform 260.2.i.b, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 260
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[131, [1, 0]], [157, [1, 0]], [41, [0, -1]]]
aps_data = [[0, 0], [-1, 1], [1, 0], [0, 1], [-3, 3], [3, -4], [0, 3], [0, 7], [3, -3], [-3, 3], [-4, 0], [7, -7], [9, -9], [0, -11], [0, 0], [-6, 0], [0, 3], [0, -11], [7, -7], [0, 3], [2, 0], [8, 0], [-12, 0], [-15, 15], [0, 7], [9, -9], [8, 0], [-9, 9], [2, 0], [0, -9], [19, -19], [-12, 0], [0, 15], [0, -5], [0, 21], [8, 0], [-10, 0], [0, 1], [3, -3], [0, 3], [21, -21], [2, 0], [0, 3], [-5, 5], [-21, 21], [0, -17], [-11, 11], [19, -19], [0, -27], [-22, 0], [18, 0], [0, 0], [0, 1], [0, -21], [-9, 9], [3, -3], [0, -27], [-23, 23], [0, 19], [6, 0], [-5, 5], [0, 27], [20, 0], [24, 0], [-22, 0], [-18, 0], [0, 19], [-34, 0], [0, 33], [1, -1], [-9, 9], [-24, 0], [-5, 5], [0, 31], [1, -1], [0, 9], [18, 0], [0, -5], [33, -33], [0, 25], [9, -9], [2, 0], [9, -9], [0, -29], [-11, 11], [0, 0], [0, -3], [-5, 5], [0, -27], [-4, 0], [-36, 0], [-39, 39], [0, -11], [9, -9], [32, 0], [0, -15], [9, -9], [30, 0], [-29, 29], [-22, 0], [8, 0], [39, -39], [0, -39], [-27, 27], [-40, 0], [2, 0], [-33, 33], [-6, 0], [24, 0], [-35, 35], [0, 13], [43, -43], [0, -33], [44, 0], [0, -17], [0, -27], [0, -11], [-45, 45], [39, -39], [0, 39], [1, -1], [-17, 17], [-42, 0], [0, -51], [-23, 23], [18, 0], [0, 37], [0, -9], [-52, 0], [-34, 0], [-47, 47], [-21, 21], [13, -13], [-29, 29], [0, -3], [13, -13], [0, 27], [0, 37], [0, 51], [-15, 15], [32, 0], [-15, 15], [0, 13], [-12, 0], [-11, 11], [0, -21], [-22, 0], [-30, 0], [44, 0], [24, 0], [0, -41], [-27, 27], [-4, 0], [-9, 9], [19, -19], [-12, 0], [0, -29], [0, -51], [2, 0], [42, 0], [-21, 21], [0, 51], [-40, 0], [0, -21], [-9, 9], [-36, 0], [61, -61], [0, 31]]
